package org.eco.als.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.ql.util.express.DefaultContext;
import com.ql.util.express.ExpressRunner;
import com.ql.util.express.InstructionSet;
import com.ql.util.express.exception.QLBizException;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.eco.als.domain.bo.ExpressBo;
import org.eco.als.service.IAlgorithmService;
import org.eco.als.service.IFormulaService;
import org.eco.als.utils.CheckVibrationDuration;
import org.eco.common.core.exception.BusinessException;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

/**
 * @author wanggaokun
 */
@Service
@Slf4j
public class FormulaService implements IFormulaService {
    @Resource
    private IAlgorithmService algorithmService;
    private ExpressRunner runner;

    @Override
    public Object evaluateExpression(ExpressBo expressBo) throws Exception {
        runner = new ExpressRunner();
        runner.addOperatorWithAlias("≠", "!=", null);
        runner.addOperatorWithAlias("≥", ">=", null);
        runner.addOperatorWithAlias("≤", "<=", null);
        // // 注册自定义(持续时间计算)函数到QLExpress上下文
        runner.addFunction("checkVibrationDuration", new CheckVibrationDuration());
        runner.setShortCircuit(true);
        Object result = null;
        try {
            runner.parseInstructionSet(expressBo.getExpression());
            JSONArray arr = algorithmService.getArrayByOssId(expressBo.getOssId());
            for (Object object : arr) {
                DefaultContext<String, Object> context = new DefaultContext<>();
                Map<String, Object> variables = CollUtil.isEmpty(expressBo.getVariables()) ? buildVariableContext(expressBo, (JSONObject) object) : expressBo.getVariables();
                context.putAll(variables);
                Boolean isMatch = (Boolean) runner.execute(expressBo.getExpression(), context, null, true, false);
                log.info("isMatch==========================:{}", isMatch);
                if (isMatch) {
                    result = object;
                    break;
                }
            }
        } catch (QLBizException e) {
            throw new BusinessException(e.getMessage());
        }
        return result;
    }

    private Map<String, Object> buildVariableContext(ExpressBo expressBo, JSONObject jsonObject) {
        String[] variableNames = parseVariableNames(expressBo.getExpression());
        Map<String, Object> variables = new HashMap<>();
        // 字段属性自定义值
        for (String fieldName : variableNames) {
            variables.put(fieldName, jsonObject.get(fieldName));
        }
        variables.put("时间", jsonObject.get("时间"));
        return variables;
    }

    /**
     * 解析表达式中的变量名。
     */
    private String[] parseVariableNames(String expression) {
        try {
            InstructionSet instructions = runner.parseInstructionSet(expression);
            return instructions.getOutAttrNames();
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
